"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (opts) {
  var analyzers = void 0;
  var libDbIdentifier = void 0;
  // Specifies the isotropy database library
  var libDbSource = t.StringLiteral("isotropy-lib-db");
  // Returns collision free identifiers for mongoConnection and the Collection
  var identifierResolver = function identifierResolver(path) {
    return {
      dbConnectionIdentifier: path.scope.generateUidIdentifier("dbConnection").name,
      collectionIdentifier: path.scope.generateUidIdentifier("collection").name
    };
  };
  /* Returns an array of two expressions
      > AssignmentExpression to mongoDB with the connectionString
      > AssignmentExpression to collection with the connectionString and collection
  */
  var mongoAccessResolver = function mongoAccessResolver(connectionString, collection, databaseConnectionIdentifier, collectionIdentifier) {
    var dbConId = (0, _babelTemplate2.default)("const " + databaseConnectionIdentifier + " = " + libDbIdentifier + ".MongoClient.connect(\"" + connectionString + "\");")();
    var colId = (0, _babelTemplate2.default)("const " + collectionIdentifier + " = db.collection(\"" + collection + "\");")();
    dbConId.declarations[0].init = t.awaitExpression(test.declarations[0].init);
    colId.declarations[0].init = t.awaitExpression(test.declarations[0].init);
    return [dbConId, colId];
  };
  return {
    plugin: {
      pre: function pre() {
        analyzers = (0, _dist2.default)();
      },

      visitor: {
        ImportDeclaration: {
          exit: function exit(path, state) {
            var analysis = analyzers.meta.analyzeImportDeclaration(path, state);
            if (!analysis) return;
            libDbIdentifier = path.scope.generateUidIdentifier("isotropyMongoDb").name;
            /*
            Inserts two statements:
              * isotropy db lib module import
            */
            path.replaceWith(t.importDeclaration([t.importDefaultSpecifier(t.identifier(libDbIdentifier))], libDbSource));
            path.skip();
          }
        },

        AssignmentExpression: {
          exit: function exit(path, state) {
            var analysis = analyzers.write.analyzeAssignmentExpression(path, state);
            if (!analysis) return;

            var _identifierResolver = identifierResolver(path),
                collectionIdentifier = _identifierResolver.collectionIdentifier,
                dbConnectionIdentifier = _identifierResolver.dbConnectionIdentifier;

            var _translate = (0, _translator2.default)(analysis.value),
                connectionString = _translate.connectionString,
                collection = _translate.collection,
                translation = _translate.translation;

            var translatedNode = babylon.parseExpression(collectionIdentifier + "." + translation);

            // const test = mongoAccessResolver(
            //   connectionString,
            //   collection,
            //   dbConnectionIdentifier,
            //   collectionIdentifier
            // );

            path.replaceWith(t.awaitExpression(translatedNode));

            path.skip();
          }
        },

        MemberExpression: {
          exit: function exit(path, state) {
            debugger;
            var analysis = analyzers.read.analyzeMemberExpression(path, state);
            if (!analysis) return;

            var _identifierResolver2 = identifierResolver(path),
                collectionIdentifier = _identifierResolver2.collectionIdentifier,
                dbConnectionIdentifier = _identifierResolver2.dbConnectionIdentifier;

            var _translate2 = (0, _translator2.default)(analysis.value),
                connectionString = _translate2.connectionString,
                collection = _translate2.collection,
                translation = _translate2.translation;

            var translatedNode = babylon.parseExpression(collectionIdentifier + "." + translation);

            // const test = mongoAccessResolver(
            //   connectionString,
            //   collection,
            //   dbConnectionIdentifier,
            //   collectionIdentifier
            // );

            path.replaceWith(t.awaitExpression(translatedNode));

            path.skip();
          }
        },

        CallExpression: {
          exit: function exit(path, state) {
            var analysis = analyzers.read.analyzeCallExpression(path, state);
            if (!analysis) return;

            var _identifierResolver3 = identifierResolver(path),
                collectionIdentifier = _identifierResolver3.collectionIdentifier,
                dbConnectionIdentifier = _identifierResolver3.dbConnectionIdentifier;

            var _translate3 = (0, _translator2.default)(analysis.value),
                connectionString = _translate3.connectionString,
                collection = _translate3.collection,
                translation = _translate3.translation;

            var translatedNode = babylon.parseExpression(collectionIdentifier + "." + translation);

            // const test = mongoAccessResolver(
            //   connectionString,
            //   collection,
            //   dbConnectionIdentifier,
            //   collectionIdentifier
            // );

            path.replaceWith(t.awaitExpression(translatedNode));

            path.skip();
          }
        }
      }
    }
  };
};

var _dist = require("../../isotropy-ast-analyzer-db/dist");

var _dist2 = _interopRequireDefault(_dist);

var _translator = require("./translator");

var _translator2 = _interopRequireDefault(_translator);

var _babelTypes = require("babel-types");

var t = _interopRequireWildcard(_babelTypes);

var _babelTemplate = require("babel-template");

var _babelTemplate2 = _interopRequireDefault(_babelTemplate);

var _babelGenerator = require("babel-generator");

var _babelGenerator2 = _interopRequireDefault(_babelGenerator);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var babylon = require("babylon");
//# sourceMappingURL=transform-to-isotropy-mongodb.js.map